// Simple PWM example on the Tiva C Series launchpad
// Will modulate the brightness of the three LED's
#include "tilm4f120hqr.h"

#define MAX_DUTY 2047
void initPWM()
{
// LEDs are on PF1,PF2,PF3
// PF1 is connected to T0CCP1 (Timer 0)
// PF2 is connected to T1CCP0 (Timer 1)
// PF3 is connected to T1CCP1 (Timer 1)
// Turn on clock for GPIOF
	SYSCTL_RCGCGPIO |= BIT5;
	SYSCTL_GPIOHBCTL |= BIT5;      // turn on AHB access to GPIOF	
// Turn on Timer 0 and Timer 1
	SYSCTL_RCGCTIMER |= BIT0 + BIT1;
// Configure the GPIO pins appropriately
// Make bits 1,2,3 outputs 
	GPIOF_DIR |= BIT1 + BIT2 + BIT3;
// Turn on alternative function (function 7) for the pins
	GPIOF_AFSEL |= BIT1 + BIT2 + BIT3;
	GPIOF_PCTL &= ~((7 << 4) + (7 << 8) + (7 << 12));
	GPIOF_PCTL |= ((7 << 4) + (7 << 8) + (7 << 12));
	GPIOF_DEN |= BIT1+BIT2+BIT3;
// Configure the Timers
//  Set preload value
	GPTM0_32_CTL = 0; // start by disabling the timers
	GPTM0_32_CFG = 4; // select 16 bit mode
	GPTM1_32_CTL = 0; // start by disabling the timers
	GPTM1_32_CFG = 4; // select 16 bit mode
	
	// Set T0AMS=1, T0ACMR = 0; T0AMR=2;
	// Set bit 10 to force reload of register to align with 
	// end of PWM period (avoids glitches)
	GPTM0_32_TAMR = BIT3+BIT1+BIT10;
	// Set T0BMS=1, T0BCMR = 0; T0BMR=2;
	GPTM0_32_TBMR = BIT3+BIT1+BIT10;	
	// Set T1AMS=1, T1ACMR = 0; T1AMR=2;
	GPTM1_32_TAMR = BIT3+BIT1+BIT10;;
	// Set T1BMS=1, T1BCMR = 0; T1BMR=2;
	GPTM1_32_TBMR = BIT3+BIT1+BIT10;;	

// Set period and initial duty
	GPTM0_32_TAILR = MAX_DUTY+1;
	GPTM0_32_TBILR = MAX_DUTY+1;
	GPTM0_32_TAMATCHR = 0;
	GPTM0_32_TBMATCHR = 0;
	GPTM1_32_TAILR = MAX_DUTY+1;
	GPTM1_32_TBILR = MAX_DUTY+1;
	GPTM1_32_TAMATCHR = 0;
	GPTM1_32_TBMATCHR = 0;	
	
	// enable T0A and T0B with output inversion
	GPTM0_32_CTL |= BIT0+BIT8+BIT6+BIT14;
	// enable T1A and T1B with output inversion
	GPTM1_32_CTL |= BIT0+BIT8+BIT6+BIT14;
	
}
void setDuty(int channel, int Duty)
{
	switch(channel)
	{
		case 0:{
			GPTM0_32_TBMATCHR = Duty;
			break;
		}
		case 1:{
			GPTM1_32_TAMATCHR = Duty;
			break;
		}
		case 2:{
			GPTM1_32_TBMATCHR = Duty;
			break;
		}
	}
}
void delay(int len)
{
	while(len--);
}
int main()
{
	int red,green,blue;
	int state;
	red = green = blue = 0;
	state = 0;
	initPWM();
	while(1)
	{
		switch(state)
		{
			case 0:
			{	
				red++;
				if (red > MAX_DUTY)
				{
					red = 1;
					state++;
				}
				setDuty(0,red);	
				break;
			}
			case 1:
			{
				green++;
				if (green > MAX_DUTY)
				{
					green = 1;
					state++;
				}
				setDuty(1,green);	
				break;
			}
			case 2:
			{
				blue++;
				if (blue > MAX_DUTY)
				{
					blue = 11;
					state = 0;
				}
				setDuty(2,blue);	
				break;
				
			}
		}
		delay(10000);
	}
}
